var ArrowLine = function(id, params) {
	this._id = id;
	this._params = params;
	this._obj = $("#" + id);
	this.init();
};

ArrowLine.prototype = {
	init : function() {
		initReferences(this);
		var me = this;
		setTimeout(function() {
			updateElementAndReferences(me);
		}, 0);
	},
	updateUI : function(obj) {
		var params = this._params;
		var width = obj.width();
		var height = obj.height();
		var borderWidth = params.bdw_0;
		var halfBdWidth = Math.ceil(borderWidth / 2);
		var type = params.type;
		var baseOffset = params.baseOffset;
		var offsetLeft = params.offsetLeft * width;
		var offsetTop = params.offsetTop * height;
		var offsetRight = params.offsetRight * width;
		var offsetBottom = params.offsetBottom * height;
		
		// create canvas
		canvas = createCanvas(width + offsetLeft + offsetRight + baseOffset * 2, height + offsetTop + offsetBottom + baseOffset * 2);
		obj.css("overflow", "visible").empty().append(canvas);
		$(canvas).css({
			"position" : "absolute",
			"left" : -(baseOffset + offsetLeft) + "px",
			"top" : -(baseOffset + offsetTop) + "px"
		});
		
		var ctx = getCanvasContext(canvas, params);
		
		var p1, p2;
		switch (type) {
			case 0:
				p1 = { x : halfBdWidth, y : (height - borderWidth) / 2 };
				p2 = { x : width - 1 - halfBdWidth, y : (height - borderWidth) / 2 };
			break;
			case 1:
				p1 = { x : halfBdWidth, y : halfBdWidth };
				p2 = { x : width - 1 - halfBdWidth, y : height - 1 - halfBdWidth };
			break;
			case 2:
				p1 = { x : width - 1 - halfBdWidth, y : halfBdWidth };
				p2 = { x : halfBdWidth, y : height - 1 - halfBdWidth };
			break;
			case 3:
				p1 = { x : (width - borderWidth) / 2, y : halfBdWidth };
				p2 = { x : (width - borderWidth) / 2, y : height - 1 - halfBdWidth };
			break;
		}
		p1.x += baseOffset;
		p1.y += baseOffset;
		p2.x += baseOffset;
		p2.y += baseOffset;
		ctx.strokeStyle = params.bdc_0;
		ctx.lineWidth = params.bdw_0;
		
		var paintStartArrow = (params.startArrow != 0);
		var paintEndArrow = (params.endArrow != 0);
		var arcRadian = params.lineArc;
		var radianSign = arcRadian == 0 ? 0 : (arcRadian > 0 ? 1 : -1);
		var dist = Math.sqrt(Math.pow(p1.x - p2.x, 2) + Math.pow(p1.y - p2.y, 2));
		var r = dist / 2 / Math.sin(Math.abs(arcRadian) * 0.00872664625);
		if (Math.abs(dist / r) < 0.3) {
			drawSwingLine(ctx, p1, p2, params.bdsw_int_0, params.bdsw_amp_0, false, true);
			// draw arrows
			if (paintStartArrow) {
				var startAngle = this._findLineAngle(p1.x, p1.y, p2.x, p2.y) + Math.PI;
				this._drawArrow(ctx, borderWidth, p1.x, p1.y, startAngle);
			}
			if (paintEndArrow) {
				var endAngle = this._findLineAngle(p1.x, p1.y, p2.x, p2.y);
				this._drawArrow(ctx, borderWidth, p2.x, p2.y, endAngle);
			}
		} else {
			// circle center
			var d1 = dist / 2;
			var h = Math.sqrt(Math.pow(r, 2) - Math.pow(d1, 2));
			var x3 = p1.x + (d1 * (p2.x - p1.x)) / dist;
			var y3 = p1.y + (d1 * (p2.y - p1.y)) / dist;
			var cx1 = x3 + (h * (p2.y - p1.y)) / dist;
			var cy1 = y3 - (h * (p2.x - p1.x)) / dist;
			var cx2 = x3 - (h * (p2.y - p1.y)) / dist;
			var cy2 = y3 + (h * (p2.x - p1.x)) / dist;
			var centerPoint = (arcRadian > 0) ? { x : cx1, y : cy1 } : { x : cx2, y : cy2 };
			
			// start and end arc
			var cos_a1 = (p1.x - centerPoint.x) / r;
			var sin_a1 = (-p1.y + centerPoint.y) / r;
			var startArc = 0;
			if (cos_a1 < 0) { 		// 2nd or 3rd region
				if (sin_a1 >= 0) {	// 2nd region
					startArc = Math.acos(cos_a1);
				} else {			// 3th region
					startArc = 2 * Math.PI - Math.acos(cos_a1);
				}
			} else {				// 1st or 4th region
				if (sin_a1 >= 0) {	// 1st region
					startArc = Math.asin(sin_a1);
				} else {			// 4th region
					startArc = 2 * Math.PI + Math.asin(sin_a1);
				}
			}
			var endArc = startArc + arcRadian * 0.0174532925;
			
			// draw the arc
			var counterClock = radianSign > 0;
			ctx.arc(centerPoint.x + offsetLeft, centerPoint.y + offsetTop, r, 2 * Math.PI - startArc, 2 * Math.PI - endArc, counterClock);
		    ctx.stroke();
			
			// draw arrows
			if (paintStartArrow) {
				var startAngle = startArc - radianSign * Math.PI / 2;
				this._drawArrow(ctx, borderWidth, p1.x + offsetLeft, p1.y + offsetTop, -startAngle);
			}
			if (paintEndArrow) {
				var endAngle = endArc + radianSign * Math.PI / 2;
				this._drawArrow(ctx, borderWidth, p2.x + offsetLeft, p2.y + offsetTop, -endAngle);
			}
		}
	},
	_findLineAngle : function(x1, y1, x2, y2) {
		if ((x2 - x1) == 0) {
			return Math.PI / 2;
		}
		var angle = Math.atan((y2 - y1) / (x2 - x1));
		if (angle < 0) {
			angle += Math.PI;
		}
		return angle;
	},
	_drawArrow : function(ctx, thickness, x0, y0, angle) {
		ctx.save();
		ctx.translate(x0, y0);
		ctx.rotate(angle);
		ctx.beginPath();
	    ctx.moveTo(0, 0);
	    ctx.lineTo(-10, -5);
	    ctx.lineTo(-10, 5);
	    ctx.closePath();
	    ctx.lineWidth = thickness;
	    ctx.lineJoin = "miter";
	    ctx.stroke();
	    ctx.fillStyle = ctx.strokeStyle;
	    ctx.fill();
		ctx.restore();
	}
};