var Balloon = function(id, params) {
	this._id = id;
	this._params = params;
	this._obj = $("#" + id);
	this.init();
};

Balloon.prototype = {
	init : function() {
		var params = this._params;
		this._text = this._obj.html();
		this._obj.empty();
		this._bg = $("<div class=\"bg\"></div>").appendTo(this._obj);
		this._fg = $("<div class=\"fg\">" + htmlToView(this._text) + "</div>").css({
			"padding" : params.margin + "px",
			"text-align" : params.txt_a_0
		}).appendTo(this._obj);
		applyFontParameters(0, "", this._fg, params);
		
		initReferences(this);
		var me = this;
		setTimeout(function() {
			updateElementAndReferences(me);
		}, 0);
	},
	updateUI : function(obj) {
		var params = this._params;
		var _bg = obj.children(".bg");
		var _fg = obj.children(".fg");
		var width = obj.width();
		var height = obj.height();
		
		// background, border and text positioning
		var radiusTopLeft = params.bdr_tl_0;
		var radiusTopRight = params.bdr_tr_0;
		var radiusBottomRight = params.bdr_br_0;
		var radiusBottomLeft = params.bdr_bl_0;
		if (params.shadow) {
			obj.css("overflow", "visible");
			_bg.css("overflow", "visible");
		}
		canvas = createCanvas(width, height, params);
		_bg.empty().append(canvas);
		var ctx = getCanvasContext(canvas, params);
		ctx.beginPath();
		var bdw = params.bdw_0;
		var swingInterval = params.bdsw_int_0;
		var swingAmplitude = params.bdsw_amp_0;
		switch (params.dir) {
			case 0:
				var netWidth = width - radiusTopLeft - radiusTopRight;
				var tipLength = height * params.tipLength; 
				var rootThickness = netWidth * params.rootWidth;
				var rootOffset = netWidth * params.rootOffset;
				if (rootOffset + rootThickness > netWidth) {
					rootOffset = netWidth - rootThickness;
				}
				var tipOffset = netWidth * params.tipOffset;
				_fg.css({
					"top" : tipLength + "px",
					"width" : (width - params.margin * 2) + "px",
					"height" : (height - tipLength) + "px"
				});
				var start = { x : bdw + radiusTopLeft, y : bdw + tipLength };
				var end = { x : bdw + radiusTopLeft + rootOffset, y : start.y };
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, false);
				start = end;
				end = { x : tipOffset, y : bdw };
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, true);
				start = end;
				end = { x : bdw + radiusTopLeft + rootOffset + rootThickness, y : bdw + tipLength };
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, true);
				start = end;
				end = { x : width - bdw - radiusTopRight, y : start.y };
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, true);
				var centerX = end.x;
				var centerY = end.y + radiusTopRight;
				ctx.arc(centerX, centerY, radiusTopRight, 1.5 * Math.PI, 0, false);
				start = { x : centerX + radiusTopRight, y : centerY };
				end = { x : width - bdw, y : height - bdw - radiusBottomRight };
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, true);
				centerX = end.x - radiusBottomRight;
				centerY = end.y;
				ctx.arc(centerX, centerY, radiusBottomRight, 0, 0.5 * Math.PI, false);
				start = { x : centerX, y : centerY + radiusBottomRight };
				end = { x : bdw + radiusBottomLeft, y : start.y };
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, true);
				centerX = end.x;
				centerY = end.y - radiusBottomLeft;
				ctx.arc(centerX, centerY, radiusBottomLeft, 0.5 * Math.PI, Math.PI, false);
				start = { x : centerX - radiusBottomLeft, y : centerY };
				end = { x : start.x, y : bdw + tipLength + radiusTopLeft };
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, true);
				centerX = end.x + radiusTopLeft;
				centerY = end.y;
				ctx.arc(centerX, centerY, radiusTopLeft, Math.PI, 1.5 * Math.PI, false);
			break;
			case 1:
				var netHeight = height - radiusTopLeft - radiusBottomLeft;
				var tipLength = width * params.tipLength;
				var rootThickness = netHeight * params.rootWidth;
				var rootOffset = netHeight * params.rootOffset;
				if (rootOffset + rootThickness > netHeight) {
					rootOffset = netHeight - rootThickness;
				}
				var tipOffset = netHeight * params.tipOffset;
				_fg.css({
					"left" : tipLength + "px",
					"width" : (width - tipLength - params.margin * 2) + "px",
					"height" : height + "px"
				});
				var start = { x : bdw + tipLength + radiusTopLeft, y : bdw };
				var end = { x : width - bdw - radiusTopRight, y : start.y };
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, false);
				var centerX = end.x;
				var centerY = end.y + radiusTopRight;
				ctx.arc(centerX, centerY, radiusTopRight, 1.5 * Math.PI, 0, false);
				start = { x : centerX + radiusTopRight, y : centerY };
				end = { x : width - bdw, y : height - bdw - radiusBottomRight };
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, true);
				centerX = end.x - radiusBottomRight;
				centerY = end.y;
				ctx.arc(centerX, centerY, radiusBottomRight, 0, 0.5 * Math.PI, false);
				start = { x : centerX, y : centerY + radiusBottomRight };
				end = { x : bdw + tipLength + radiusBottomLeft, y : start.y };
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, true);
				centerX = end.x;
				centerY = end.y - radiusBottomLeft;
				ctx.arc(centerX, centerY, radiusBottomLeft, 0.5 * Math.PI, Math.PI, false);
				start = { x : centerX - radiusBottomLeft, y : centerY };
				end = { x : start.x, y : bdw + radiusTopLeft + rootOffset + rootThickness };
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, true);
				start = end;
				end = { x : bdw, y : tipOffset };
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, true);
				start = end;
				end = { x : bdw + tipLength, y : bdw + radiusTopLeft + rootOffset };
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, true);
				start = end;
				end = { x : start.x, y : bdw + radiusTopLeft };
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, true);
				centerX = end.x + radiusTopLeft;
				centerY = end.y;
				ctx.arc(centerX, centerY, radiusTopLeft, Math.PI, 1.5 * Math.PI, false);
			break;
			case 2:
				var netWidth = width - radiusBottomLeft - radiusBottomRight;
				var tipLength = height * params.tipLength; 
				var rootThickness = netWidth * params.rootWidth;
				var rootOffset = netWidth * params.rootOffset;
				if (rootOffset + rootThickness > netWidth) {
					rootOffset = netWidth - rootThickness;
				}
				var tipOffset = netWidth * params.tipOffset;
				_fg.css({
					"width" : (width - params.margin * 2) + "px",
					"height" : (height - tipLength) + "px"
				});
				var start = { x : bdw + radiusTopLeft, y : bdw };
				var end = { x : width - bdw - radiusTopRight, y : start.y };
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, false);
				var centerX = end.x;
				var centerY = end.y + radiusTopRight;
				ctx.arc(centerX, centerY, radiusTopRight, 1.5 * Math.PI, 0, false);
				start = { x : centerX + radiusTopRight, y : centerY };
				end = { x : width - bdw, y : height - bdw - tipLength - radiusBottomRight };
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, true);
				centerX = end.x - radiusBottomRight;
				centerY = end.y;
				ctx.arc(centerX, centerY, radiusBottomRight, 0, 0.5 * Math.PI, false);
				start = { x : centerX, y : centerY + radiusBottomRight };
				end = { x : bdw + radiusBottomLeft + rootOffset + rootThickness, y : start.y };
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, true);
				start = end;
				end = { x : tipOffset, y : height - bdw };
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, true);
				start = end;
				end = { x : bdw + radiusBottomLeft + rootOffset, y : height - bdw - tipLength};
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, true);
				start = end;
				end = { x : bdw + radiusBottomLeft, y : start.y};
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, true);
				centerX = end.x;
				centerY = end.y - radiusBottomLeft;
				ctx.arc(centerX, centerY, radiusBottomLeft, 0.5 * Math.PI, Math.PI, false);
				start = { x : centerX - radiusBottomLeft, y : centerY };
				end = { x : start.x, y : bdw + radiusTopLeft };
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, true);
				centerX = end.x + radiusTopLeft;
				centerY = end.y;
				ctx.arc(centerX, centerY, radiusTopLeft, Math.PI, 1.5 * Math.PI, false);
			break;
			case 3:
				var netHeight = height - radiusTopRight - radiusBottomRight;
				var tipLength = width * params.tipLength;
				var rootThickness = netHeight * params.rootWidth;
				var rootOffset = netHeight * params.rootOffset;
				if (rootOffset + rootThickness > netHeight) {
					rootOffset = netHeight - rootThickness;
				}
				tipOffset = netHeight * params.tipOffset;
				_fg.css({
					"width" : (width - tipLength - params.margin * 2) + "px",
					"height" : height + "px"
				});
				var start = { x : bdw + radiusTopLeft, y : bdw };
				var end = { x : width - bdw - tipLength - radiusTopRight, y : start.y };
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, false);
				var centerX = end.x;
				var centerY = end.y + radiusTopRight;
				ctx.arc(centerX, centerY, radiusTopRight, 1.5 * Math.PI, 0, false);
				start = { x : centerX + radiusTopRight, y : centerY };
				end = { x : start.x, y : bdw + radiusTopRight + rootOffset };
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, true);
				start = end;
				end = { x : width - bdw, y : tipOffset };
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, true);
				start = end;
				end = { x : width - bdw - tipLength, y : bdw + rootOffset + rootThickness };
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, true);
				start = end;
				end = { x : start.x, y : height - bdw - radiusBottomRight };
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, true);
				centerX = end.x - radiusBottomRight;
				centerY = end.y;
				ctx.arc(centerX, centerY, radiusBottomRight, 0, 0.5 * Math.PI, false);
				start = { x : centerX, y : centerY + radiusBottomRight };
				end = { x : bdw + radiusBottomLeft, y : start.y };
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, true);
				centerX = end.x;
				centerY = end.y - radiusBottomLeft;
				ctx.arc(centerX, centerY, radiusBottomLeft, 0.5 * Math.PI, Math.PI, false);
				start = { x : centerX - radiusBottomLeft, y : centerY };
				end = { x : start.x, y : bdw + radiusTopLeft };
				drawSwingLine(ctx, start, end, swingInterval, swingAmplitude, true);
				centerX = end.x + radiusTopLeft;
				centerY = end.y;
				ctx.arc(centerX, centerY, radiusTopLeft, Math.PI, 1.5 * Math.PI, false);
			break;
		}
		ctx.closePath();
		fillAndStrokeOnCanvas(ctx, width, height, params);
	}
};