var BarChart = function(id, params) {
	this._id = id;
	this._params = params;
	this._obj = $("#" + id);
	this.init();
};

BarChart.prototype = {
	init : function() {
		initReferences(this);
		var me = this;
		setTimeout(function() {
			updateElementAndReferences(me);
		}, 0);
	},
	updateUI : function(obj) {
		var params = this._params;
		var width = obj.width();
		var height = obj.height();
		var opaque = params.bgo_0 > 0;
		
		if (params.shadow) {
			obj.css("overflow", "visible");	
		}

		// create canvas
		var canvas = createCanvas(width, height, opaque ? undefined : params);
		obj.empty().append(canvas);
		var ctx = opaque ? canvas.getContext('2d') : getCanvasContext(canvas, params);
		
		// draw background
		ctx.beginPath();
		ctx.rect(0, 0, width, height);
		fillAndStrokeOnCanvas(ctx, width, height, params, false, true);
		
		// draw border
		ctx.beginPath();
		var borderColor = params.bdc_0;
		var borderWidthTop = params.bd_t_0;
		var borderWidthRight = params.bd_r_0;
		var borderWidthBottom = params.bd_b_0;
		var borderWidthLeft = params.bd_l_0;
		var swingInterval = params.bdsw_int_0;
		var swingAmplitude = params.bdsw_amp_0;
		var offset = Math.floor(swingAmplitude / 2);
		ctx.strokeStyle = borderColor;
		drawSwingBorder(ctx, offset, offset, width - 2 * offset, height - 2 * offset, borderWidthTop, borderWidthRight, borderWidthBottom, borderWidthLeft, 0, 0, 0, 0, swingInterval, swingAmplitude);
		
		// draw bars
		var barValues = params.values;
		var maxValue = params.maxVal;
		var barColors = params.colors;
		var bdColors = params.bdColors;
		var margins = params.margins;
		var barCount = barValues.length;
		var bdWidth = params.bdw_0;
		
		switch (params.dir) {
			case 0:	// base bottom
				var chartWidth = width - margins.left - margins.right;
				var chartHeight = height - margins.top;
				var barThickness = chartWidth / barCount;
				var ratio = chartHeight / maxValue;
				var curX = margins.left;
				for (var i = 0; i < barCount; i ++) {
					var barColor = barColors[i];
					var bdColor = bdColors[i];
					var barLength = barValues[i] * ratio;
					ctx.fillStyle = barColor;
					ctx.rect(curX, height - 1 - barLength, barThickness, barLength);
					ctx.fill();
					ctx.strokeStyle = bdColor;
					drawSwingBorder(ctx, curX, height - 1 - barLength, barThickness, barLength, bdWidth, bdWidth, bdWidth, bdWidth, 0, 0, 0, 0, params.bdsw_int_0, params.bdsw_amp_0);
					curX += barThickness;
				}
			break;
			case 1:	// base left
				var chartWidth = width - margins.right;
				var chartHeight = height - margins.top - margins.bottom;
				var barThickness = chartHeight / barCount;
				var ratio = chartWidth / maxValue;
				var curY = margins.top;
				for (var i = 0; i < barCount; i ++) {
					var barColor = barColors[i];
					var bdColor = bdColors[i];
					var barLength = barValues[i] * ratio;
					ctx.fillStyle = barColor;
					ctx.rect(bdWidth, curY, barLength, barThickness);
					ctx.fill();
					ctx.strokeStyle = bdColor;
					drawSwingBorder(ctx, bdWidth, curY, barLength, barThickness, bdWidth, bdWidth, bdWidth, bdWidth, 0, 0, 0, 0, params.bdsw_int_0, params.bdsw_amp_0);
					curY += barThickness;
				}
			break;
			case 2:	// base top
				var chartWidth = width - margins.left - margins.right;
				var chartHeight = height - margins.top;
				var barThickness = chartWidth / barCount;
				var ratio = chartHeight / maxValue;
				var curX = margins.left;
				for (var i = 0; i < barCount; i ++) {
					var barColor = barColors[i];
					var bdColor = bdColors[i];
					var barLength = barValues[i] * ratio;
					ctx.fillStyle = barColor;
					ctx.rect(curX, bdWidth, barThickness, barLength);
					ctx.fill();
					ctx.strokeStyle = bdColor;
					drawSwingBorder(ctx, curX, bdWidth, barThickness, barLength, bdWidth, bdWidth, bdWidth, bdWidth, 0, 0, 0, 0, params.bdsw_int_0, params.bdsw_amp_0);
					curX += barThickness;
				}
			break;
			case 3:	// base right
				var chartWidth = width - margins.right;
				var chartHeight = height - margins.top - margins.bottom;
				var barThickness = chartHeight / barCount;
				var ratio = chartWidth / maxValue;
				var curY = margins.top;
				for (var i = 0; i < barCount; i ++) {
					var barColor = barColors[i];
					var bdColor = bdColors[i];
					var barLength = barValues[i] * ratio;
					ctx.fillStyle = barColor;
					ctx.rect(width - 1 - barLength, curY, barLength, barThickness);
					ctx.fill();
					ctx.strokeStyle = bdColor;
					drawSwingBorder(ctx, width - 1 - barLength, curY, barLength, barThickness, bdWidth, bdWidth, bdWidth, bdWidth, 0, 0, 0, 0, params.bdsw_int_0, params.bdsw_amp_0);
					curY += barThickness;
				}
			break;
		}
		
		// shadow
		if (opaque) {
			applyShadow(obj, $(canvas), params.shadow);
		}
	}
};