var Menu = function(id, params) {
	this._id = id;
	this._params = params;
	this._obj = $("#" + id);
	this.init();
};

Menu.prototype = {
	init : function() {
		var params = this._params;
		var content = this._obj.children();
		this._obj.empty();
		this._bg = $("<div class=\"bg\"></div>")
			.appendTo(this._obj);
		this._bd = $("<div class=\"bd\"></div>")
			.appendTo(this._obj);
		this._fg = $("<div class=\"fg\"></div>")
			.appendTo(this._obj);
		
		var me = this;
		var rowHeight = params.rowh;
		var gap = params.gap;
		var icons = params.icons;
		var margin = params.txtm;
		var menu = $("<table style=\"position:absolute;\" cellspacing=\"0\" cellpadding=\"0\"></table>").appendTo(this._fg);
		content.children("li").each(function(index) {
			var text = $(this).html();
			if (me._isSeparator(text)) {
				$("<tr class=\"menuItem\"><td class=\"menuCell separator\" style=\"height:3px;font-size:1px;\"></td></tr>").appendTo(menu);
			} else {
				var parts = text.split(",");
				var label = createLabelObject(icons[index], 0, gap, parts[0]);
				var menuCell = $("<td valign=\"middle\" class=\"menuCell\"></td>").append(label).width("100%").height(rowHeight);
				var menuItem = $("<tr class=\"menuItem\"></tr>").append(menuCell).appendTo(menu);
				if (parts[1]) {
					var icon = false;
					var txt = parts[1];
					if (me._isExpandArrow(parts[1])) {
						txt = "";
						icon = params.expandIcon_0;
						menuItem.addClass("expandable");
					}
					var floatLabel = createLabelObject(icon, 0, 0, txt, true).css({
						"position" : "relative",
						"float" : "right"
					}).appendTo(menuCell);
					floatLabel.find(".icon").addClass("expandIcon");
				}
				menuItem.children().css({
					"padding-left" : margin + "px",
					"padding-right" : margin + "px",
					"cursor" : "default"
				});
			}
		});
		
		params.hlIndex = -1;
		
		this.setupBehavior(this._obj);
		initReferences(this);

		setTimeout(function() {
			updateElementAndReferences(me);
		}, 0);
	},
	updateUI : function(obj) {
		var me = this;
		var params = this._params;
		var _bg = obj.children(".bg");
		var _bd = obj.children(".bd");
		var _fg = obj.children(".fg");
		var width = obj.width();
		var height = obj.height();

		// menu size
		var borderWidth = params.bdw_0;
		var menuTable = _fg.children("table");
		menuTable.css({
			left : borderWidth + "px",
			top : borderWidth + "px",
			width : (width - borderWidth * 2) + "px",
			height : (height - borderWidth * 2) + "px"
		});
		
		// background and border
		_bd.css("border-width", "0px").width(obj.width()).height(obj.height());
		generateBackgroundAndBorders(0, "", _bg, _bd, params);

		// add padding rows
		var rowCount = params.rows;
		var rowHeight = params.rowh;
		
		var paddingRowHeight = height - rowCount * rowHeight - borderWidth;
		var menuBody = menuTable.children();
		menuBody.children(".paddingRow").remove();
		if (paddingRowHeight > 0) {
			menuBody.append($("<tr class=\"menuItem paddingRow\"><td class=\"menuCell\"></td></tr>").height(paddingRowHeight));
		}
		
		// prepare rows in bg
		_bg.children("table").remove();
		var bgRows = menuTable.clone().appendTo(_bg).find(".menuCell").css({
			"padding-left" : "0px",
			"padding-right" : "0px"
		}).html("");
		
		// prepare rows in bd
		_bd.children("table").remove();
		var bdRows = menuTable.clone().appendTo(_bd).find(".menuCell").html("");

		// tweak rows
		var margin = params.txtm;
		var selRow = params.selr;
		var hlIndex = params.hlIndex;
		var gap = params.gap;
		var icons = params.icons;
		menuBody.children(".menuItem").each(function(index) {
			var menuItem = $(this);
			var fgCell = menuItem.children();
			var bgCell = $(bgRows.get(index));
			var bdCell = $(bdRows.get(index));
			var tarId = (index == hlIndex || (hlIndex == -1 && index == selRow)) ? "sel" : "item";
			generateBackgroundAndBorders(0, tarId, bgCell, bdCell, params);
			if (!menuItem.hasClass("paddingRow")) {
				if (fgCell.hasClass("separator")) {
					fgCell.children("canvas").remove();
					fgCell.children(".line").remove();
					if (params.bdsw_amp_0 == 0) {
						$("<div class=\"line\"></div>").css({
							"border-style" : "solid",
							"border-width" : "0px",
							"border-bottom-width" : params.bdw_0 + "px",
							"border-color" : params.bdc_0,
							"width" : "100%",
							"height" : "1px"
						}).appendTo(fgCell);
					} else {
						var w = fgCell.width();
						var h = fgCell.height();
						var canvas = createCanvas(w, h);
						var ctx = canvas.getContext('2d');
						ctx.strokeStyle = params.bdc_0;
						ctx.lineWidth = params.bdw_0;
						drawSwingLine(ctx, { x : 3, y : 1 }, { x : w - 3, y : 1}, params.bdsw_int_0, params.bdsw_amp_0, false, true);
						fgCell.append(canvas);
					}
				} else {
					var bkTextColor = params[tarId + "f_c_" + 0];
					if (params.disabledRows[index]) {
						params[tarId + "f_c_" + 0] = shadeColor(params[tarId + "f_c_" + 0], 0.6);
					}
					applyFontParameters(0, tarId, fgCell, params);
					params[tarId + "f_c_" + 0] = bkTextColor;
					var preferredWidth = getTextSize(fgCell.text(), params.itemf_n_0, params.itemf_s_0).width;
					if (icons[index]) {
						preferredWidth += gap + fgCell.find(".icon").width();
					}
					fgCell.css("padding-left", "0px");
					var actualMargin = Math.max(0, Math.min(margin, (width - preferredWidth) / 2));
					fgCell.css("padding-left", actualMargin + "px")
						.find(".expandIcon").attr("src", "images/" + (tarId == "sel" ? params.hlExpandIcon_0 : params.expandIcon_0));
				}
			}
		});
		
		// shadow
		applyShadow(obj, _bd, params.shadow);
	},
	_isSeparator : function(text) {
		return /^[\s-=]*$/.test(text);
	},
	_isExpandArrow : function(text) {
		return /^\s*(&gt;)+\s*$/.test(text);
	},
	setupBehavior : function(obj) {
		var me = this;
		var params = this._params;
		var items = obj.children(".fg").find(".menuItem");
		items.click(function() {
			var menuItem = $(this);
			var curIndex = items.index(this);
			if (!params.disabledRows[curIndex]) {
				if (menuItem.hasClass("expandable")) {
					$(me).trigger("Expand_Menu", [me._id, items.index(this)]);
				} else {
					if (!menuItem.children(".menuCell").hasClass("separator")) {
						me.setSelectedIndex(curIndex + 1);
						var parentMenu = obj.parent().closest("[id^='MultilevelMenu_'],[ref^='MultilevelMenu_']");
						if (parentMenu.size() > 0) {
							parentMenu.css("visibility", "hidden").trigger("Hidden");
						} else {
							obj.css("visibility", "hidden").trigger("Hidden");
						}
					}
				}
			}
		}).hover(function() {
			if (!$(this).children(".menuCell").hasClass("separator")) {
				var curIndex = items.index(this);
				if (params.hlIndex != curIndex) {
					params.hlIndex = params.disabledRows[curIndex] ? -1 : curIndex;
					updateElementAndReferences(me);
				}
			}
		}, function() {
			params.hlIndex = -1;
			updateElementAndReferences(me);
		});
		obj.mouseleave(function() {
			params.selr = -1;
			params.hlIndex = -1;
			updateElementAndReferences(me);
		});
		$(document).mousedown(function(event) {
			var eventSrc = $(event.target);
			var menu = eventSrc.closest("[id^='Menu_'],[ref^='Menu_']");
			var parentMenu = eventSrc.closest("[id^='MultilevelMenu_'],[ref^='MultilevelMenu_']");
			if (menu.size() == 0 && parentMenu.size() == 0) {
				obj.css("visibility", "hidden").trigger("Hidden");
			}
		});
	},
	getSelectedIndex : function() {
		return this._params.selr + 1;
	},
	setSelectedIndex : function(index) {
		var params = this._params;
		if (params.selr != index - 1) {
			params.selr = index - 1;
			updateElementAndReferences(this);
			$("#" + this._id).trigger("Selection_Changed");
		}
	}
};