var PieChart = function(id, params) {
	this._id = id;
	this._params = params;
	this._obj = $("#" + id);
	this.init();
};

PieChart.prototype = {
	init : function() {
		initReferences(this);
		var me = this;
		setTimeout(function() {
			updateElementAndReferences(me);
		}, 0);
	},
	updateUI : function(obj) {
		var params = this._params;
		var width = obj.width();
		var height = obj.height();
		var hwr = height / width;
		var opaque = params.bgo_0 > 0;
		
		if (params.shadow) {
			obj.css("overflow", "visible");	
		}

		// create canvas
		var canvas = createCanvas(width, height, opaque ? undefined : params);
		obj.empty().append(canvas);
		var ctx = opaque ? canvas.getContext('2d') : getCanvasContext(canvas, params);
		
		// draw background
		ctx.beginPath();
		ctx.rect(0, 0, width, height);
		fillAndStrokeOnCanvas(ctx, width, height, params, false, true);
		
		// draw border
		ctx.beginPath();
		var borderColor = params.bdc_0;
		var borderWidthTop = params.bd_t_0;
		var borderWidthRight = params.bd_r_0;
		var borderWidthBottom = params.bd_b_0;
		var borderWidthLeft = params.bd_l_0;
		var swingInterval = params.bdsw_int_0;
		var swingAmplitude = params.bdsw_amp_0;
		var offset = Math.floor(swingAmplitude / 2);
		ctx.strokeStyle = borderColor;
		drawSwingBorder(ctx, offset, offset, width - 2 * offset, height - 2 * offset, borderWidthTop, borderWidthRight, borderWidthBottom, borderWidthLeft, 0, 0, 0, 0, swingInterval, swingAmplitude);
		
		// draw pieces
		var colors = params.colors;
		var pieceValues = params.values;
		var pieceCount = pieceValues.length;
		var pieceColors = params.colors;
		var bdColors = params.bdColors;
		var margins = params.margins;
		var pieceTotalValue = 0;
		for (var i = 0; i < pieceCount; i ++) {
			pieceTotalValue += pieceValues[i];
		}
		if (pieceCount > 0) {
			var radius = (width - margins.left - margins.right) / 2;
			var baseArc = 0;
			for (var i = 0; i < pieceCount; i ++) {
				var curArc = 360 - pieceValues[i] * 360 / pieceTotalValue;
				var x = (width + margins.left - margins.right) / 2;
				var y = (height + margins.top - margins.bottom) / 2
				ctx.save();
				ctx.translate(x, y);
				ctx.scale(1, hwr);
				ctx.beginPath();
				ctx.moveTo(0, 0);
				ctx.arc(0, 0, radius, baseArc * 0.0174532925, (baseArc + curArc) * 0.0174532925, true);
				ctx.closePath();
				ctx.restore();
				ctx.fillStyle = pieceColors[i];
				ctx.strokeStyle = bdColors[i];
				ctx.fill();
				ctx.stroke();
				baseArc += curArc;
			}
		}
		
		// shadow
		if (opaque) {
			applyShadow(obj, $(canvas), params.shadow);
		}
	}
};