var Tree = function(id, params) {
	this._id = id;
	this._params = params;
	this._obj = $("#" + id);
	this.init();
};

Tree.prototype = {
	init: function() {
		var params = this._params;
		var content = this._obj.children();
		this._obj.empty();
		this._bg = $("<div class=\"bg\"></div>")
			.appendTo(this._obj);
		this._bd = $("<div class=\"bd\"></div>")
			.appendTo(this._obj);
		this._fg = $("<div class=\"fg\"></div>")
			.appendTo(this._obj);

		var me = this;
		var state = params.state;
		var paintRootHandles = params["rthd_" + state];
		var iconTextGap = params.gap;
		var textMargin = params.txtm;
		var rowHeights = params.rowh;
		var borderWidth = params["bdw_" + state];
		var indentWidth = params.indentWidth;
		this._fg.css({
			"padding" : borderWidth + "px"
		});
		content.children("li").each(function(index) {
			var rowHtml = $(this).html();
			var indents = /^(\t*)/.exec(rowHtml);
			var indentCount = indents == null ? 0 : indents[0].length;
			rowHtml = rowHtml.slice(indentCount);
			var img = /^\s*\[img(\d+)\]/.exec(rowHtml);
			if (img != null) {
				var imgId = img[1];
				var imgHtml = "<img class=\"icon\" src=\"images/dock/" + imgId + "." + window.foreui.imagetypes[imgId] + "\" align=\"absmiddle\" style=\"margin-right:" + iconTextGap + "px;\" />";
				rowHtml = imgHtml + "<span class=\"treeNodeText\">" + rowHtml.slice(img[0].length) + "</span>";
			} else {
				rowHtml = "<span class=\"treeNodeText\">" + rowHtml + "</span>";
			}
			var rowHeight = rowHeights[index];
			var treeRow = $("<div class=\"treeRow\" indent=\"" + indentCount + "\"><div class=\"treeRowBg\" style=\"width:100%;height:" + rowHeight + "px\"></div><table cellspacing=\"0\" cellpadding=\"0\" style=\"width:100%;height:100%;position:relative;top:-" + rowHeight + "px;display:block;\"><tr style=\"height:" + rowHeight + "px;\"><td class=\"treeNode\">" + rowHtml + "</td></tr></table></div>")
				.width("100%").height(rowHeight).appendTo(me._fg);
			var indentBoxWidth = params.indent * indentCount + (paintRootHandles ? params.indent : textMargin);
			var indentBox = $("<td class=\"indentBox\" valign=\"top\"></td>").width(indentBoxWidth);
			if (indentCount > 0 || paintRootHandles) {
				var lineBlockUp = $("<div class=\"lineUp\"></div>").width(indentWidth / 2 - borderWidth).height(rowHeight / 2).css({
					"position" : "absolute",
					"left" : (indentBoxWidth - indentWidth / 2) + "px",
					"top" : "0px",
					"border-style" : "solid",
					"border-width" : "0px",
					"border-left-width" : borderWidth + "px",
					"border-bottom-width" : borderWidth + "px"
				});
				var lineBlockDown = $("<div class=\"lineDown\"></div>").width(indentWidth / 2 - borderWidth).height(Math.ceil(rowHeight / 2)).css({
					"position" : "absolute",
					"left" : (indentBoxWidth - indentWidth / 2) + "px",
					"top" : (rowHeight / 2) + "px",
					"border-style" : "solid",
					"border-width" : "0px",
					"border-left-width" : borderWidth + "px",
					"border-top-width" : borderWidth + "px"
				});
				indentBox.append(lineBlockUp).append(lineBlockDown);
			}
			var startPaintLineIndent = paintRootHandles ? 0 : 1;
			for (var i = startPaintLineIndent; i < indentCount; i ++) {
				var lineBlock = $("<div class=\"line" + (indentCount - 1 - i + startPaintLineIndent) + "\"></div>").width(indentWidth).height(rowHeight).css({
					"position" : "absolute",
					"left" : (indentBoxWidth - indentWidth * (i - startPaintLineIndent + 1.5)) + "px",
					"top" : "0px",
					"border-style" : "solid",
					"border-width" : "0px",
					"border-left-width" : borderWidth + "px"
				});
				indentBox.append(lineBlock);
			}
			var toggle = $("<div class=\"toggle\"><img /></div>").css({
				"position" : "absolute",
				"left" : (indentBoxWidth - indentWidth / 2 - 4) + "px",
				"top" : (rowHeight / 2 - 4) + "px",
				"font-size" : "1px",
				"cursor" : "pointer"
			});
			indentBox.append(toggle);
			treeRow.children().children().children("tr").prepend(indentBox);
		});
		
		this.setupBehavior(this._obj);
		initReferences(this);

		setTimeout(function() {
			setState(me, state, true);
			updateElementAndReferences(me);
		}, 0);
	},
	updateUI: function(obj) {
		var me = this;
		var params = this._params;
		var state = params.state;
		var _bg = obj.children(".bg");
		var _bd = obj.children(".bd");
		var _fg = obj.children(".fg");
		
		var borderType = params["bdt_" + state];
		var borderWidth = params["bdw_" + state];
		var borderReduce = borderType == "swing" ? "0" : borderWidth;
		var swingAmplitude = params["bdsw_amp_" + state];
		var width = obj.width() - borderWidth * 2 - swingAmplitude;
		var height = obj.height() - borderWidth * 2 - swingAmplitude;
		
		var paintBorder = params["pbdr_" + state];
		var paintLines = params["lines_" + state];
		var paintDefaultIcons = params["dicon_" + state];
		var paintRootHandles = params["rthd_" + state];
		
		// background and border
		_bd.css("border-width", "0px").width(obj.width()).height(obj.height());
		if (!paintBorder) {
			params["bdt_" + state] = "none";
		}
		generateBackgroundAndBorders(state, "", _bg, _bd, params);
		
		// collapse or expand the rows
		var rowCollapsedMap = params.collapsed;
		for (index in rowCollapsedMap) {
			this._collapseOrExpandRow(obj, parseInt(index), rowCollapsedMap[index] ? true : false);
		}
		
		// append padding rows
		_fg.children(".paddingRow").remove();
		var defaultRowHeight = params.rh;
		var rowHeights = params.rowh;
		var paddingHeight = height;
		_fg.children(".treeRow").each(function(index) {
			paddingHeight -= ($(this).css("display") == "none" ? 0 : (rowHeights[index] ? rowHeights[index] : 0));
		});
		paddingHeight = Math.max(0, paddingHeight);
		var paddingRowCount = Math.ceil(paddingHeight / defaultRowHeight);
		for (var i = 0; i < paddingRowCount; i ++) {
			_fg.append($("<div class=\"treeRow paddingRow\"></div>").width("100%").height(defaultRowHeight));
		}
		
		// tweak rows
		var borderColor = params["bdc_" + state];
		var iconTextGap = params.gap;
		var shownIndex = 0;
		_fg.children(".treeRow").each(function(index) {
			var row = $(this).width(width);
			
			// background and border
			if (row.css("display") != "none") {
				var rowBg = row.children(".treeRowBg");
				if (rowBg.size() == 0) {
					rowBg = row;
				}
				rowBg.css("background-color", "").children(".linearBg").remove();
				var treeNodeText = row.find(".treeNodeText");
				treeNodeText.css("background-color", "").children(".linearBg").remove();
				if (index == params.selr) {
					generateBackgroundAndBorders(state, "sel", rowBg, row, params);
					generateBackgroundAndBorders(state, "seln", treeNodeText, treeNodeText, params);
					treeNodeText.children(".linearBg").css("position", "absolute").css("z-index", "-1");
				} else if (shownIndex % 2 == 1) {
					generateBackgroundAndBorders(state, "alt", rowBg, row, params);
				}
				rowBg.children(".linearBg").css("position", "absolute");
				shownIndex ++;
			}
			
			if (!row.hasClass("paddingRow")) {
				var node = row.children().children().children().children(".treeNode");
				node.css({
					"white-space" : "nowrap",
					"cursor" : "default"
				});
				
				// font
				applyFontParameters(state, params.selr == index ? "seln" : "", node, params);
				
				// default icon
				var hasChild = me._hasChild(index);
				var isCollapsed = rowCollapsedMap[index] ? true : false;
				node.children(".defaultIcon").remove();
				if (paintDefaultIcons && node.children(".icon").size() == 0) {
					var iconPrefix = hasChild ? (isCollapsed ? "closedIcon_" : "openedIcon_") : "nodeIcon_";
					var iconSrc = params[iconPrefix + state];
					var icon = $("<img class=\"defaultIcon\" src=\"images/" + iconSrc + "\" align=\"absmiddle\" style=\"margin-right:" + iconTextGap + "px;\" />");
					node.prepend(icon);
				}
				
				// adjust lines
				var lineColorCss = { "border-color" : borderColor };
				var indent = parseInt(row.attr("indent"));
				var indentBox = row.children().children().children("tr").children(".indentBox");
				indentBox.find(".lineUp").css("visibility", index == 0 || !paintLines ? "hidden" : "inherit").css(lineColorCss);
				indentBox.find(".lineDown").css("visibility", me._hasNextSibling(index) && paintLines ? "inherit" : "hidden").css(lineColorCss);
				for (var i = indent - 1; i >= 0; i --) {
					indentBox.find(".line" + i).css("visibility", (paintLines && me._hasUpperRowWithIndent(index, i) && me._hasLowerRowWithIndent(index, i)) ? "inherit" : "hidden").css(lineColorCss);
				}
				
				// toggle
				var toggle = indentBox.children(".toggle").css("visibility", hasChild ? "inherit" : "hidden");
				toggle.children("img").attr("src", "images/" + params[(isCollapsed ? "expandableIcon_" : "collapsableIcon_") + state]);
			}
		});
		
		// shadow
		applyShadow(obj, _bd, params.shadow);
	},
	setupBehavior : function(obj) {
		var me = this;
		var params = this._params;
		var _fg = obj.children(".fg");
		var rows = _fg.children(".treeRow:not(.paddingRow)");
		rows.click(function() {
			var index = rows.index(this);
			if (params.selr != index) {
				params.selr = index;
				updateElementAndReferences(me);
				$("#" + me._id).trigger("Selection_Changed");
			}
		});
		var toggles = rows.children().children().children("tr").children(".indentBox").children(".toggle");
		var rowCollapsedMap = params.collapsed;
		toggles.click(function(event) {
			var index = toggles.index(this);
			var isCollapsed = rowCollapsedMap[index] ? true : false;
			rowCollapsedMap[index] = !isCollapsed;
			updateElementAndReferences(me);
			event.stopPropagation();
		});
	},
	_hasChild : function(row) {
		var treeRows = this._obj.children(".fg").children(".treeRow:not(.paddingRow)");
		var treeRow = $(treeRows.get(row));
		if (treeRow.hasClass("paddingRow") || row >= treeRows.size() - 1 || row < 0) {
			return false;
		}
		var curIndent = parseInt(treeRow.attr("indent"));
		var nextIndent = parseInt($(treeRows.get(row + 1)).attr("indent"));
		return (nextIndent == curIndent + 1);
	},
	_hasNextSibling : function(row) {
		var treeRows = this._obj.children(".fg").children(".treeRow:not(.paddingRow)");
		var treeRow = $(treeRows.get(row));
		var rowIndent = parseInt(treeRow.attr("indent"));
		var rowCount = treeRows.size();
		for (var i = row + 1; i < rowCount; i ++) {
			var curIndent = parseInt($(treeRows.get(i)).attr("indent"));
			if (curIndent == rowIndent) {
				return true;
			}
			if (curIndent < rowIndent) {
				break;
			}
		}
		return false;
	},
	_hasUpperRowWithIndent : function(row, indent) {
		var treeRows = this._obj.children(".fg").children(".treeRow:not(.paddingRow)");
		for (var i = row - 1; i >= 0; i --) {
			var curIndent = parseInt($(treeRows.get(i)).attr("indent"));
			if (curIndent == indent) {
				return true;
			}
		}
		return false;
	},
	_hasLowerRowWithIndent : function(row, indent) {
		var treeRows = this._obj.children(".fg").children(".treeRow:not(.paddingRow)");
		var rowCount = treeRows.size();
		for (var i = row + 1; i < rowCount; i ++) {
			var curIndent = parseInt($(treeRows.get(i)).attr("indent"));
			if (curIndent == indent) {
				return true;
			}
		}
		return false;
	},
	_collapseOrExpandRow : function(obj, row, collapsed) {
		var treeRows = obj.children(".fg").children(".treeRow:not(.paddingRow)");
		var treeRow = $(treeRows.get(row));
		var rowIndent = parseInt(treeRow.attr("indent"));
		var rowCount = treeRows.size();
		for (var i = row + 1; i < rowCount; i ++) {
			var curRow = $(treeRows.get(i));
			var curIndent = parseInt(curRow.attr("indent"));
			if (curIndent <= rowIndent) {
				break;
			}
			curRow.css("display", collapsed ? "none" : "block");
		}
	},
	getRowCount : function() {
		return this._obj.children(".fg").children(".treeRow:not(.paddingRow)").size();
	},
	getSelectedIndex : function() {
		return this._params.selr + 1;
	},
	setSelectedIndex : function(index) {
		var tarIndex = index - 1;
		if (this._params.selr != tarIndex) {
			this._params.selr = tarIndex;
			updateElementAndReferences(this);
			$("#" + this._id).trigger("Selection_Changed");
		}
	},
	setNodeValue : function(targets, row, value) {
		for (var i = 0; i < targets.length; i ++) {
			var elem = getForeUIElementById(targets[i]);
			if (elem && elem.getRowCount() >= row) {
				$("[ref=\"" + elem._id + "\"]").add(elem._obj).children(".fg").each(function() {
					var node = $(this).children(".treeRow").eq(row - 1).children().children().children().children(".treeNode");
					var icon = node.children(".icon,.defaultIcon").remove();
					node.text(htmlToView(value)).prepend(icon);
				});
			}
		}
	},
	getSelectedText : function() {
		var index = this._params.selr;
		if (index == -1) {
			return "";
		}
		var node = this._fg.children(".treeRow").eq(index).children().children().children().children(".treeNode");
		return textFromView(node.text());
	}
};